@@ -1,6 +1,6 @@
 import os
-import shutil
 import subprocess
+import sys
 from pathlib import Path
 from urllib.parse import urlparse
 
@@ -12,6 +12,7 @@ from ctfcli.utils.challenge import (
     create_challenge,
     lint_challenge,
     load_challenge,
+    load_installed_challenge,
     load_installed_challenges,
     sync_challenge,
 )
@@ -24,6 +25,7 @@ from ctfcli.utils.config import (
 from ctfcli.utils.deploy import DEPLOY_HANDLERS
 from ctfcli.utils.spec import CHALLENGE_SPEC_DOCS, blank_challenge_spec
 from ctfcli.utils.templates import get_template_dir
+from ctfcli.utils.git import get_git_repo_head_branch
 
 
 class Challenge(object):
@@ -51,27 +53,53 @@ class Challenge(object):
 
         Templates().list()
 
-    def add(self, repo):
+    def add(self, repo, yaml_path=None):
         config = load_config()
 
         if repo.endswith(".git"):
             # Get relative path from project root to current directory
             challenge_path = Path(os.path.relpath(os.getcwd(), get_project_path()))
 
-            # Get new directory that will exist after clone
+            # Get new directory that will add the git subtree
             base_repo_path = Path(os.path.basename(repo).rsplit(".", maxsplit=1)[0])
 
             # Join targets
             challenge_path = challenge_path / base_repo_path
             print(challenge_path)
 
-            config["challenges"][str(challenge_path)] = repo
+            # If a custom yaml_path is specified we add it to our challenge_key
+            if yaml_path:
+                challenge_key = str(challenge_path / yaml_path)
+            else:
+                challenge_key = str(challenge_path)
 
+            config["challenges"][challenge_key] = repo
+
+            head_branch = get_git_repo_head_branch(repo)
+            subprocess.call(
+                [
+                    "git",
+                    "subtree",
+                    "add",
+                    "--prefix",
+                    challenge_path,
+                    repo,
+                    head_branch,
+                    "--squash",
+                ],
+                cwd=get_project_path(),
+            )
             with open(get_config_path(), "w+") as f:
                 config.write(f)
 
-            subprocess.call(["git", "clone", "--depth", "1", repo])
-            shutil.rmtree(str(base_repo_path / ".git"))
+            subprocess.call(
+                ["git", "add", ".ctf/config"], cwd=get_project_path(),
+            )
+            subprocess.call(
+                ["git", "commit", "-m", f"Added {str(challenge_path)}"],
+                cwd=get_project_path(),
+            )
+
         elif Path(repo).exists():
             config["challenges"][repo] = repo
             with open(get_config_path(), "w+") as f:
@@ -89,9 +117,21 @@ class Challenge(object):
             if url.endswith(".git"):
                 if challenge is not None and folder != challenge:
                     continue
-                click.echo(f"Cloning {url} to {folder}")
-                subprocess.call(["git", "clone", "--depth", "1", url, folder])
-                shutil.rmtree(str(Path(folder) / ".git"))
+                click.echo(f"Adding git repo {url} to {folder} as subtree")
+                head_branch = get_git_repo_head_branch(url)
+                subprocess.call(
+                    [
+                        "git",
+                        "subtree",
+                        "add",
+                        "--prefix",
+                        folder,
+                        url,
+                        head_branch,
+                        "--squash",
+                    ],
+                    cwd=get_project_path(),
+                )
             else:
                 click.echo(f"Skipping {url} - {folder}")
 
@@ -162,7 +202,7 @@ class Challenge(object):
                     break
             else:
                 click.secho(
-                    f'Couldn\'t find existing challenge {c["name"]}. Perhaps you meant install instead of sync?',
+                    f'Couldn\'t find existing challenge {challenge["name"]}. Perhaps you meant install instead of sync?',
                     fg="red",
                 )
                 continue  # Go to the next challenge in the overall list
@@ -178,22 +218,24 @@ class Challenge(object):
             if challenge and challenge != folder:
                 continue
             if url.endswith(".git"):
-                click.echo(f"Cloning {url} to {folder}")
-                subprocess.call(["git", "init"], cwd=folder)
-                subprocess.call(["git", "remote", "add", "origin", url], cwd=folder)
-                subprocess.call(["git", "add", "-A"], cwd=folder)
+                click.echo(f"Pulling latest {url} to {folder}")
+                head_branch = get_git_repo_head_branch(url)
                 subprocess.call(
-                    ["git", "commit", "-m", "Persist local changes (ctfcli)"],
-                    cwd=folder,
-                )
-                subprocess.call(
-                    ["git", "pull", "--allow-unrelated-histories", "origin", "master"],
-                    cwd=folder,
+                    [
+                        "git",
+                        "subtree",
+                        "pull",
+                        "--prefix",
+                        folder,
+                        url,
+                        head_branch,
+                        "--squash",
+                    ],
+                    cwd=get_project_path(),
                 )
                 subprocess.call(["git", "mergetool"], cwd=folder)
                 subprocess.call(["git", "clean", "-f"], cwd=folder)
                 subprocess.call(["git", "commit", "--no-edit"], cwd=folder)
-                shutil.rmtree(str(Path(folder) / ".git"))
             else:
                 click.echo(f"Skipping {url} - {folder}")
 
@@ -299,3 +341,76 @@ class Challenge(object):
             click.secho(
                 f"An error occured during deployment", fg="red",
             )
+
+    def push(self, challenge=None):
+        config = load_config()
+        challenges = dict(config["challenges"])
+        if challenge is None:
+            # Get relative path from project root to current directory
+            challenge_path = Path(os.path.relpath(os.getcwd(), get_project_path()))
+            challenge = str(challenge_path)
+
+        try:
+            url = challenges[challenge]
+            head_branch = get_git_repo_head_branch(url)
+            subprocess.call(
+                ["git", "subtree", "push", "--prefix", challenge, url, head_branch],
+                cwd=get_project_path(),
+            )
+        except KeyError:
+            click.echo(
+                "Couldn't process that challenge path. Please check that the challenge is added to .ctf/config and that your path matches."
+            )
+
+    def healthcheck(self, challenge):
+        config = load_config()
+        challenges = config["challenges"]
+
+        # challenge_path = challenges[challenge]
+        path = Path(challenge)
+        if path.name.endswith(".yml") is False:
+            path = path / "challenge.yml"
+
+        challenge = load_challenge(path)
+        click.secho(f'Loaded {challenge["name"]}', fg="yellow")
+        try:
+            healthcheck = challenge["healthcheck"]
+        except KeyError:
+            click.secho(f'{challenge["name"]} missing healthcheck parameter', fg="red")
+            return
+
+        # Get challenges installed from CTFd and try to find our challenge
+        installed_challenges = load_installed_challenges()
+        target = None
+        for c in installed_challenges:
+            if c["name"] == challenge["name"]:
+                target = c
+                break
+        else:
+            click.secho(
+                f'Couldn\'t find challenge {c["name"]} on CTFd', fg="red",
+            )
+            return
+
+        # Get the actual challenge data
+        installed_challenge = load_installed_challenge(target["id"])
+        connection_info = installed_challenge["connection_info"]
+
+        # Run healthcheck
+        if connection_info:
+            rcode = subprocess.call(
+                [healthcheck, "--connection-info", connection_info], cwd=path.parent
+            )
+        else:
+            rcode = subprocess.call([healthcheck], cwd=path.parent)
+
+        if rcode != 0:
+            click.secho(
+                f"Healcheck failed", fg="red",
+            )
+            sys.exit(1)
+        else:
+            click.secho(
+                f"Success", fg="green",
+            )
+            sys.exit(0)
