@@ -5,6 +5,7 @@ import click
 import yaml
 
 from .config import generate_session
+from .tools import strings
 
 
 class Yaml(dict):
@@ -23,6 +24,11 @@ def load_challenge(path):
         return
 
 
+def load_installed_challenge(challenge_id):
+    s = generate_session()
+    return s.get(f"/api/v1/challenges/{challenge_id}", json=True).json()["data"]
+
+
 def load_installed_challenges():
     s = generate_session()
     return s.get("/api/v1/challenges?view=admin", json=True).json()["data"]
@@ -46,6 +52,9 @@ def sync_challenge(challenge, ignore=[]):
     if challenge.get("attempts") and "attempts" not in ignore:
         data["max_attempts"] = challenge.get("attempts")
 
+    if challenge.get("connection_info") and "connection_info" not in ignore:
+        data["connection_info"] = challenge.get("connection_info")
+
     data["state"] = "hidden"
 
     installed_challenges = load_installed_challenges()
@@ -76,14 +85,38 @@ def sync_challenge(challenge, ignore=[]):
                 r.raise_for_status()
         for flag in challenge["flags"]:
             if type(flag) == str:
-                data = {"content": flag, "type": "static", "challenge": challenge_id}
+                data = {"content": flag, "type": "static", "challenge_id": challenge_id}
                 r = s.post(f"/api/v1/flags", json=data)
                 r.raise_for_status()
             elif type(flag) == dict:
-                flag["challenge"] = challenge_id
+                flag["challenge_id"] = challenge_id
                 r = s.post(f"/api/v1/flags", json=flag)
                 r.raise_for_status()
 
+    # Update topics
+    if challenge.get("topics") and "topics" not in ignore:
+        # Delete existing challenge topics
+        current_topics = s.get(
+            f"/api/v1/challenges/{challenge_id}/topics", json=""
+        ).json()["data"]
+        for topic in current_topics:
+            topic_id = topic["id"]
+            r = s.delete(
+                f"/api/v1/topics?type=challenge&target_id={topic_id}", json=True
+            )
+            r.raise_for_status()
+        # Add new challenge topics
+        for topic in challenge["topics"]:
+            r = s.post(
+                f"/api/v1/topics",
+                json={
+                    "value": topic,
+                    "type": "challenge",
+                    "challenge_id": challenge_id,
+                },
+            )
+            r.raise_for_status()
+
     # Update tags
     if challenge.get("tags") and "tags" not in ignore:
         # Delete existing tags
@@ -94,7 +127,9 @@ def sync_challenge(challenge, ignore=[]):
                 r = s.delete(f"/api/v1/tags/{tag_id}", json=True)
                 r.raise_for_status()
         for tag in challenge["tags"]:
-            r = s.post(f"/api/v1/tags", json={"challenge": challenge_id, "value": tag})
+            r = s.post(
+                f"/api/v1/tags", json={"challenge_id": challenge_id, "value": tag}
+            )
             r.raise_for_status()
 
     # Upload files
@@ -119,7 +154,7 @@ def sync_challenge(challenge, ignore=[]):
                 click.secho(f"File {file_path} was not found", fg="red")
                 raise Exception(f"File {file_path} was not found")
 
-        data = {"challenge": challenge_id, "type": "challenge"}
+        data = {"challenge_id": challenge_id, "type": "challenge"}
         # Specifically use data= here instead of json= to send multipart/form-data
         r = s.post(f"/api/v1/files", files=files, data=data)
         r.raise_for_status()
@@ -136,12 +171,12 @@ def sync_challenge(challenge, ignore=[]):
 
         for hint in challenge["hints"]:
             if type(hint) == str:
-                data = {"content": hint, "cost": 0, "challenge": challenge_id}
+                data = {"content": hint, "cost": 0, "challenge_id": challenge_id}
             else:
                 data = {
                     "content": hint["content"],
                     "cost": hint["cost"],
-                    "challenge": challenge_id,
+                    "challenge_id": challenge_id,
                 }
 
             r = s.post(f"/api/v1/hints", json=data)
@@ -193,6 +228,9 @@ def create_challenge(challenge, ignore=[]):
     if challenge.get("attempts") and "attempts" not in ignore:
         data["max_attempts"] = challenge.get("attempts")
 
+    if challenge.get("connection_info") and "connection_info" not in ignore:
+        data["connection_info"] = challenge.get("connection_info")
+
     s = generate_session()
 
     r = s.post("/api/v1/challenges", json=data)
@@ -205,7 +243,7 @@ def create_challenge(challenge, ignore=[]):
     if challenge.get("flags") and "flags" not in ignore:
         for flag in challenge["flags"]:
             if type(flag) == str:
-                data = {"content": flag, "type": "static", "challenge": challenge_id}
+                data = {"content": flag, "type": "static", "challenge_id": challenge_id}
                 r = s.post(f"/api/v1/flags", json=data)
                 r.raise_for_status()
             elif type(flag) == dict:
@@ -213,10 +251,25 @@ def create_challenge(challenge, ignore=[]):
                 r = s.post(f"/api/v1/flags", json=flag)
                 r.raise_for_status()
 
+    # Create topics
+    if challenge.get("topics") and "topics" not in ignore:
+        for topic in challenge["topics"]:
+            r = s.post(
+                f"/api/v1/topics",
+                json={
+                    "value": topic,
+                    "type": "challenge",
+                    "challenge_id": challenge_id,
+                },
+            )
+            r.raise_for_status()
+
     # Create tags
     if challenge.get("tags") and "tags" not in ignore:
         for tag in challenge["tags"]:
-            r = s.post(f"/api/v1/tags", json={"challenge": challenge_id, "value": tag})
+            r = s.post(
+                f"/api/v1/tags", json={"challenge_id": challenge_id, "value": tag}
+            )
             r.raise_for_status()
 
     # Upload files
@@ -231,7 +284,7 @@ def create_challenge(challenge, ignore=[]):
                 click.secho(f"File {file_path} was not found", fg="red")
                 raise Exception(f"File {file_path} was not found")
 
-        data = {"challenge": challenge_id, "type": "challenge"}
+        data = {"challenge_id": challenge_id, "type": "challenge"}
         # Specifically use data= here instead of json= to send multipart/form-data
         r = s.post(f"/api/v1/files", files=files, data=data)
         r.raise_for_status()
@@ -240,12 +293,12 @@ def create_challenge(challenge, ignore=[]):
     if challenge.get("hints") and "hints" not in ignore:
         for hint in challenge["hints"]:
             if type(hint) == str:
-                data = {"content": hint, "cost": 0, "challenge": challenge_id}
+                data = {"content": hint, "cost": 0, "challenge_id": challenge_id}
             else:
                 data = {
                     "content": hint["content"],
                     "cost": hint["cost"],
-                    "challenge": challenge_id,
+                    "challenge_id": challenge_id,
                 }
 
             r = s.post(f"/api/v1/hints", json=data)
@@ -335,4 +388,19 @@ def lint_challenge(path):
     if errored:
         exit(1)
 
+    # Check that files don't have a flag in them
+    files = challenge.get("files", [])
+    errored = False
+    for f in files:
+        fpath = Path(path).parent / f
+        for s in strings(fpath):
+            # TODO make flag format customizable
+            if "flag" in s:
+                print(
+                    f"Potential flag {s} found in distributed file {fpath.absolute()}"
+                )
+                errored = True
+    if errored:
+        exit(1)
+
     exit(0)
